package com.koobietech.eas.controller;

import com.anji.captcha.model.common.ResponseModel;
import com.anji.captcha.model.vo.CaptchaVO;
import com.anji.captcha.service.CaptchaService;
import com.koobietech.eas.common.exception.EasException;
import com.koobietech.eas.common.result.JsonResult;
import com.koobietech.eas.dao.Pojo.AdminPojo;
import com.koobietech.eas.dao.adminLoginPojo.UserDetail;
import com.koobietech.eas.dao.dto.LoginToken;
import com.koobietech.eas.service.StudentLoginService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@RestController
@Tag(name = "学生登陆模块")
public class StudentLoginController {

    @Resource
    CaptchaService captchaService;

    @Resource
    StudentLoginService studentLoginService;

    @PostMapping("/studentLogin")
    @Operation(summary = "学生登陆", description = "用户名和密码为请求载荷，若登录成功，返回两个token")
    public JsonResult studentLogin(@RequestBody AdminPojo adminPojo) {

        CaptchaVO captchaVO = new CaptchaVO();
        captchaVO.setCaptchaVerification(adminPojo.getCaptchaVerification());

        ResponseModel response = captchaService.verification(captchaVO);
        if (!response.isSuccess()) {
            throw new EasException(response.getRepMsg(), Integer.parseInt(response.getRepCode()));
        }

        LoginToken ret = studentLoginService.studentLogin(adminPojo);
        return JsonResult.data(ret);
    }

    @GetMapping("/getStudentInfo")
    @Operation(summary = "获取学生信息", description = "根据Token获取学生信息")
    public JsonResult getStudentInfo(@RequestHeader("Authorization") String token) {
        UserDetail ret = studentLoginService.getStudentInfo(token);
        return JsonResult.data(ret);
    }

    @PostMapping("/studentRefreshToken")
    @Operation(summary = "刷新token", description = "当token过期，在请求头中携带refresh token，若刷新成功，返回新的token和refresh token")
    public JsonResult refreshToken(@RequestHeader("AUthorization") String refreshToken) {
        LoginToken ret = studentLoginService.refreshToken(refreshToken);
        return JsonResult.data(ret);
    }

    @PostMapping("/studentVerifyGet")
    @Operation(summary = "获取验证码", description = "前端发起获取验证码请求，后端访问静态资源返回滑动图片")
    public JsonResult verifyGet(@RequestBody CaptchaVO captchaVO) {
        JsonResult ret = studentLoginService.verifyToJsonResult(captchaService.get(captchaVO));
        return JsonResult.data(ret);
    }

    @PostMapping("/studentVerifyCheck")
    @Operation(summary = "核对验证码轨迹", description = "前端把用户的滑动轨迹转化成字符串传入后端，通过算法判断轨迹是否为真人")
    public JsonResult verifyCheck(@RequestBody CaptchaVO captchaVO) {
        JsonResult ret = studentLoginService.verifyToJsonResult(captchaService.check(captchaVO));
        return JsonResult.data(ret);
    }

}
